// @HEADER
// ***********************************************************************
//
//                 TriUtils: Trilinos Utilities Package
//                 Copyright (2011) Sandia Corporation
//
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Michael A. Heroux (maherou@sandia.gov)
//
// ***********************************************************************
// @HEADER

#include "Trilinos_Util.h"

double Trilinos_Util_smsrres (int m, int /* n */,
    double *val, int *indx,
    double *xlocal, double *x, double *b)
{
  int i, j, jbgn, jend;
  double sum, norm_tmp = 0.0, norm_b = 0.0;
  double scaled_res_norm = 0.0, res_norm, *tmp, max_norm = 0.0;


  /*     Computes the residual

         res = || b - A*x ||

         where x and b are vectors and A is a sparse matrix stored
         in MSR format. */

  /*     --------------------------
         First executable statement
         -------------------------- */

  /* Create tmp workspace */
  tmp = (double *) calloc(m,sizeof(double));

  /* .....initialize soln */

  for (i = 0; i < m; i++)
    tmp[i] = b[i] - val[i] * xlocal[i];

  /* .....do a series of SPDOTs (sparse dot products) */

  for (i = 0; i <m ; i++)
  {
    jbgn = indx[i];
    jend = indx[i + 1];
    sum = 0.0;

    for (j = jbgn; j < jend; j++)
      sum += val[j] * x[indx[j]];

    tmp[i] -= sum;
    max_norm = Trilinos_Util_max(fabs(tmp[i]),max_norm);
    norm_tmp += tmp[i]*tmp[i];
    norm_b += b[i]*b[i];
  }

  res_norm = sqrt(norm_tmp);
  printf("\n\nMax norm of residual        = %12.4g\n",max_norm);
  printf(    "Two norm of residual        = %12.4g\n",res_norm);
  if (norm_b > 1.0E-7)
  {
    scaled_res_norm = res_norm/sqrt(norm_b);
    printf(    "Scaled two norm of residual = %12.4g\n",scaled_res_norm);
  }
  free((void *) tmp);

  return(scaled_res_norm);
} /* smsrres */

